#!/usr/bin/env python

# have_I_b33n_pwned.py
# Check if your password (hash) appears in the leaked password database
# of haveibeenpwned.com
#
# Copyright (C) 2019 willipink.eu
# Author Moritz Münch moritzmuench@mailbox.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
# updater script for nextcloud
#
# TODO
# - add feature: keepass integration? isnt there something like this already?


from os import name
from sys import argv, stdout, exit
from re import match
from argparse import ArgumentParser
from hashlib import sha1
from getpass import getpass
from requests import get


if name == 'nt':  # disable colors for windows environment
    (RED, GREEN, RESET) = ("", "", "")
else:
    (RED, GREEN, RESET) = ("\033[1;31m", "\033[0;32m", "\033[0;0m")

PASSWORD_API = 'https://api.pwnedpasswords.com/range/'
MAIL_API = 'https://haveibeenpwned.com/api/v2/breachedaccount/'
ROW = '{:<30}{:<10}{:<45}'
HIDDEN = True
INFINITE = True


parser = ArgumentParser(
    description='Check if your email or password appears in a data leak.',
    epilog='Moritz Münch, moritzmuench@mailbox.org, willipink.eu, GPL3+')
parser.add_argument('input',
    nargs='*',
    metavar='password or mail adress',
    help='''You can specify passwords and mail adresses seperated by spaces.
    If the password contains a space you have to enclose it in quotes like ' or
    "''')
parser.add_argument('-s', '-show', '--show',
        nargs='?',
        help='Show entered in leak summary')
args = parser.parse_args()
if args.show:
    HIDDEN = False


def input_prompt():
    global HIDDEN
    if HIDDEN:
        user_input = getpass('Tell me your password or mail adress: ')
    else:
        user_input = input('Tell me your password or mail adress: ')
    check_mail_or_password(user_input)


def check_mail_or_password(user_input):
    valid_mail = match(r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$', user_input)
    if valid_mail:
        query_mail(valid_mail.group())
    else:
        query_password(user_input)


def query_password(password):
    ''' Query the first 5 digits of the sha1 hash of the user input
    ti the haveibeenpwned.com api. '''
    password_hash = sha1(password.encode('UTF-8')).hexdigest().upper()
    request = password_hash[:5]
    response = get(PASSWORD_API + request).text
    hash_searched = 'not yet'
    for answer in response.splitlines():
        data = answer.split(':')
        combined_hash = request + data[0]
        if password_hash == combined_hash:
            hash_searched = int(data[1])
            break
    if hash_searched == 'not yet':
        stdout.write(GREEN)
    else:
        stdout.write(RED)
    if HIDDEN:
        password = '*' * len(password)
    print(ROW.format('password', 'leaked', 'sha1'))
    print('-' * 80)
    print(ROW.format(password, hash_searched, password_hash))
    stdout.write(RESET)
    if INFINITE:
        input_prompt()


def query_mail(mail):
    try:
        response = get(MAIL_API + mail).json()
        for breach in response:
            print('Name: {}'.format(breach['Name']))
            print('Title: {}'.format(breach['Title']))
            print('Domain: {}'.format(breach['Domain']))
            print('Breach date: {}'.format(breach['BreachDate']))
            print('Added date: {}'.format(breach['AddedDate']))
            print('Modified date: {}'.format(breach['ModifiedDate']))
            print('Pwn count: {}'.format(breach['PwnCount']))
            print('Description: {}'.format(breach['Description']))
            stdout.write(RED)
            print('Breached data: {}'.format(', '.join([data for data in breach['DataClasses']])))
            stdout.write(RESET)
            print('Verified: {}'.format('yes' if breach['IsVerified'] == 1 else 'No'))
            print('Fabricated: {}'.format('yes' if breach['IsFabricated'] == 1 else 'No'))
            print('Sensitive: {}'.format('yes' if breach['IsSensitive'] == 1 else 'No'))
            print('Retired: {}'.format('yes' if breach['IsRetired'] == 1 else 'No'))
            print('Spam: {}'.format('yes' if breach['IsSpamList'] == 1 else 'No'))
    except:
        stdout.write(GREEN)
        print('The mail adress {} is not in the leak database.'.format(mail))
        stdout.write(RESET)
    query_password(mail)


if __name__ == '__main__':
    if args.input:
        for user_input in args.input:
            check_mail_or_password(user_input)
    else:
        INFINITE = True
        input_prompt()
